@@ -78,13 +78,11 @@ pub fn functions() -> HashSet<&'static str>
         "arccoth",
         "cis",
         "ln",
-        "aexp",
         "ceil",
         "floor",
         "round",
         "recip",
         "exp",
-        "aln",
         "log",
         "root",
         "bi",
@@ -93,7 +91,6 @@ pub fn functions() -> HashSet<&'static str>
         "max",
         "min",
         "sqrt",
-        "asquare",
         "abs",
         "norm",
         "re",
@@ -104,15 +101,12 @@ pub fn functions() -> HashSet<&'static str>
         "sign",
         "arg",
         "cbrt",
-        "acube",
         "frac",
         "fract",
         "int",
         "trunc",
         "square",
-        "asqrt",
         "cube",
-        "acbrt",
         "fact",
         "subfact",
         "sinc",
@@ -216,6 +210,21 @@ pub fn functions() -> HashSet<&'static str>
         "unity",
         "iter",
         "weighted",
+        "lobf",
+        "lineofbestfit",
+        "onlyreal",
+        "onlyre",
+        "ore",
+        "onlyimag",
+        "onlyim",
+        "oim",
+        "mod",
+        "cossin",
+        "sincos",
+        "surfacearea",
+        "sarea",
+        "cov",
+        "covariance",
     ]
     .iter()
     .cloned()
