@@ -8,11 +8,11 @@ use crate::{
         NumStr::{Matrix, Num, Str, Vector},
     },
     load_vars::set_commands_or_vars,
-    math::do_math,
+    math::{compute_funcvars, do_math},
     misc::{place_funcvar, place_funcvarxy, place_var, place_varxy, prompt},
     options::silent_commands,
     parse::input_var,
-    Colors, GraphType, Number, Options, Variable,
+    Colors, GraphType, HowGraphing, Number, Options, Variable,
 };
 use gnuplot::{Auto, AxesCommon, Caption, Color, Figure, Fix, PointSymbol, TickOption};
 use rug::Complex;
@@ -94,15 +94,15 @@ pub fn graph(
                     &mut 0,
                     options,
                     false,
-                    false,
                     0,
                     Vec::new(),
+                    true,
                 )
                 {
-                    Ok(f) => (f.0, f.1, options),
-                    _ =>
+                    Ok(f) => (f.0, f.1, options, f.2),
+                    Err(s) =>
                     {
-                        print!("\x1b[G\x1b[Kbad input\x1b[G\n{}", prompt(options, &colors));
+                        print!("\x1b[G\x1b[K{}\x1b[G\n{}", s, prompt(options, &colors));
                         stdout().flush().unwrap();
                         return;
                     }
@@ -232,24 +232,37 @@ pub fn graph(
             {
                 options.yr = options.vyr;
             }
-            let (xticks, yticks) = if options.ticks == -2.0
+            let xticks = if options.ticks.0 == -2.0
             {
-                (Some((Auto, 0)), Some((Auto, 0)))
+                Some((Auto, 0))
             }
-            else if options.ticks == -1.0
+            else if options.ticks.0 == -1.0
             {
-                (Some((Fix(1.0), 0)), Some((Fix(1.0), 0)))
+                Some((Fix(1.0), 0))
             }
-            else if options.ticks == 0.0
+            else if options.ticks.0 == 0.0
             {
-                (None, None)
+                None
             }
             else
             {
-                (
-                    Some((Fix((options.xr.1 - options.xr.0) / options.ticks), 0)),
-                    Some((Fix((options.yr.1 - options.yr.0) / options.ticks), 0)),
-                )
+                Some((Fix((options.xr.1 - options.xr.0) / options.ticks.0), 0))
+            };
+            let yticks = if options.ticks.1 == -2.0
+            {
+                Some((Auto, 0))
+            }
+            else if options.ticks.1 == -1.0
+            {
+                Some((Fix(1.0), 0))
+            }
+            else if options.ticks.1 == 0.0
+            {
+                None
+            }
+            else
+            {
+                Some((Fix((options.yr.1 - options.yr.0) / options.ticks.1), 0))
             };
             let ratio = if options.scale_graph
             {
@@ -655,29 +668,53 @@ pub fn graph(
             {
                 options.zr = options.vzr;
             }
-            let (xticks, yticks, zticks) = if options.ticks == -2.0
+            let xticks = if options.ticks.0 == -2.0
             {
-                (Some((Auto, 0)), Some((Auto, 0)), Some((Auto, 0)))
+                Some((Auto, 0))
             }
-            else if options.ticks == -1.0
+            else if options.ticks.0 == -1.0
             {
-                (
-                    Some((Fix(1.0), 0)),
-                    Some((Fix(1.0), 0)),
-                    Some((Fix(1.0), 0)),
-                )
+                Some((Fix(1.0), 0))
             }
-            else if options.ticks == 0.0
+            else if options.ticks.0 == 0.0
             {
-                (None, None, None)
+                None
             }
             else
             {
-                (
-                    Some((Fix((options.xr.1 - options.xr.0) / options.ticks), 0)),
-                    Some((Fix((options.yr.1 - options.yr.0) / options.ticks), 0)),
-                    Some((Fix((options.zr.1 - options.zr.0) / options.ticks), 0)),
-                )
+                Some((Fix((options.xr.1 - options.xr.0) / options.ticks.0), 0))
+            };
+            let yticks = if options.ticks.1 == -2.0
+            {
+                Some((Auto, 0))
+            }
+            else if options.ticks.1 == -1.0
+            {
+                Some((Fix(1.0), 0))
+            }
+            else if options.ticks.1 == 0.0
+            {
+                None
+            }
+            else
+            {
+                Some((Fix((options.yr.1 - options.yr.0) / options.ticks.1), 0))
+            };
+            let zticks = if options.ticks.2 == -2.0
+            {
+                Some((Auto, 0))
+            }
+            else if options.ticks.2 == -1.0
+            {
+                Some((Fix(1.0), 0))
+            }
+            else if options.ticks.2 == 0.0
+            {
+                None
+            }
+            else
+            {
+                Some((Fix((options.zr.1 - options.zr.0) / options.ticks.2), 0))
             };
             if options.surface
                 && points3d[1..6]
@@ -1185,7 +1222,12 @@ pub fn graph(
 }
 #[allow(clippy::type_complexity)]
 pub fn get_list_2d(
-    mut func: (Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options),
+    mut func: (
+        Vec<NumStr>,
+        Vec<(String, Vec<NumStr>)>,
+        Options,
+        HowGraphing,
+    ),
 ) -> ([[Vec<f64>; 2]; 2], [Vec<f64>; 2], (bool, bool))
 {
     {
@@ -1485,7 +1527,12 @@ pub fn get_list_2d(
 }
 #[allow(clippy::type_complexity)]
 pub fn get_list_3d(
-    mut func: (Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options),
+    mut func: (
+        Vec<NumStr>,
+        Vec<(String, Vec<NumStr>)>,
+        Options,
+        HowGraphing,
+    ),
 ) -> ([[Vec<f64>; 3]; 2], (bool, bool), bool)
 {
     {
@@ -1558,6 +1605,7 @@ pub fn get_list_3d(
         let num = Num(Number::from(Complex::with_val(func.2.prec, n), None));
         modified = place_var(func.0.clone(), "x", num.clone());
         modifiedvars = place_funcvar(func.1.clone(), "x", num);
+        compute_funcvars(&mut modified, func.2, &mut modifiedvars);
         for g in 0..=func.2.samples_3d.1
         {
             let f = func.2.yr.0 + g as f64 * den_y_range;
@@ -1811,7 +1859,12 @@ fn fail(options: Options, colors: &Colors, input: String)
 #[allow(clippy::type_complexity)]
 fn get_data(
     colors: Colors,
-    func: (Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options),
+    func: (
+        Vec<NumStr>,
+        Vec<(String, Vec<NumStr>)>,
+        Options,
+        HowGraphing,
+    ),
     input: String,
 ) -> JoinHandle<(
     (bool, bool),
@@ -1828,12 +1881,7 @@ fn get_data(
         let mut points3d: [[Vec<f64>; 3]; 2] = Default::default();
         let mut d2_or_d3: (bool, bool) = (false, false);
         let mut re_or_im = (false, false);
-        let (has_x, has_y) = (
-            func.0.iter().any(|i| i.str_is("x"))
-                || func.1.iter().any(|i| i.1.iter().any(|i| i.str_is("x"))),
-            func.0.iter().any(|i| i.str_is("y"))
-                || func.1.iter().any(|i| i.1.iter().any(|i| i.str_is("y"))),
-        );
+        let (has_x, has_y) = (func.3.x, func.3.y);
         if !has_y && !has_x
         {
             match match do_math(func.0.clone(), func.2, func.1)
@@ -1923,12 +1971,14 @@ fn get_data(
                                 vec![Num(Number::from(v[0].number.clone(), None))],
                                 Vec::new(),
                                 func.2,
+                                HowGraphing::default(),
                             ));
                             let points2dtemp: [[Vec<f64>; 2]; 2];
                             (points2dtemp, _, re_or_im) = get_list_2d((
                                 vec![Num(Number::from(v[1].number.clone(), None))],
                                 Vec::new(),
                                 func.2,
+                                HowGraphing::default(),
                             ));
                             points2d[0][0].extend(points2dtemp[0][0].clone());
                             points2d[0][1].extend(points2dtemp[0][1].clone());
