@@ -22,7 +22,7 @@ FLAGS: --help (this message)\x1b[G\n\
 --range=[num] sets all ranges to [-num],[num]\x1b[G\n\
 --point [char] point style for graphing\x1b[G\n\
 --base=[input],[output] sets the numbers base from 2 to 36\x1b[G\n\
---ticks=[num] sets amount of ticks, -2 will be auto, -1 will be at every whole number, 0 will be none\x1b[G\n\
+--ticks=[num](,[num](,[num])) sets amount of ticks, optionally set different x/y/z ticks, -2 will be auto, -1 will be at every whole number, 0 will be none\x1b[G\n\
 --onaxis toggles showing the ticks on the x/y/z axis\x1b[G\n\
 --prompt toggles the prompt\x1b[G\n\
 --color toggles color output, toggled by default when running from arguments\x1b[G\n\
@@ -51,7 +51,7 @@ FLAGS: --help (this message)\x1b[G\n\
 - \"vars\" to list all variables\x1b[G\n\
 - \"option/var;function\" to set a temporal option/var, example: \"a=45;angle=deg;sin(a)\" = sqrt(2)/2\x1b[G\n\
 - \"f(x)=var:function\" to set a temporal var when defining function, example: \"f(x)=a=2:ax\" = f(x)=2x\x1b[G\n\
-- \"_\" to use the previous answer\x1b[G\n\
+- \"_\" or \"ans\" or \"ANS\" to use the previous answer\x1b[G\n\
 - \"a={{expr}}\" to define a variable\x1b[G\n\
 - \"f(x)=...\" to define a function\x1b[G\n\
 - \"f(x,y,z...)=...\" to define a multi variable function\x1b[G\n\
@@ -83,7 +83,7 @@ Order of Operations:\x1b[G\n\
 - < (lt), <= (le), > (gt), >= (ge), == (eq), != (!eq), >> (a>>b shifts b bits right), << (a<<b shifts b bits left)\x1b[G\n\
 - && (a&&b outputs 1 if both a and b are 1), || (a||b outputs 1 if either a or b are 1)\x1b[G\n\n\
 Functions:\x1b[G\n\
-- sin, cos, tan, asin, acos, atan, atan(x,y)\x1b[G\n\
+- sin, cos, tan, asin, acos, atan, atan(x,y), atan2(y,x), sincos(x)={{sin(x),cos(x)}}, cossin(x)={{cos(x),sin(x)}}\x1b[G\n\
 - csc, sec, cot, acsc, asec, acot\x1b[G\n\
 - sinh, cosh, tanh, asinh, acosh, atanh\x1b[G\n\
 - csch, sech, coth, acsch, asech, acoth\x1b[G\n\
@@ -95,14 +95,14 @@ Functions:\x1b[G\n\
 - fact, doublefact, subfact\x1b[G\n\
 - sinc, cis, exp\x1b[G\n\
 - zeta, eta, gamma, beta, erf, erfc, digamma, ai, multinomial, binomial/bi/C(n,r), P(n,r), pochhammer(x,n)\x1b[G\n\
-- re, im, split(x+yi), next(n,to)\x1b[G\n\
+- re, im, onlyreal, onlyimag, split(x+yi), next(n,to)\x1b[G\n\
 - unity(n,k) gets all solutions for x in x^k=n\x1b[G\n\
 - factors, prime, isprime, gcd, lcm\x1b[G\n\
 - slog(a,b), ssrt(k,a) (k is lambert w branch)\x1b[G\n\
 - piecewise/pw({{value,cond}},{{value2,cond2}}...) (when first condition is met from left to right. value elsewards is nan)\x1b[G\n\
 - vec(var,func,start,end) mat(var,func,start,end) (makes a vector/matrix) start..end is a shortcut to vec(n,n,start,end)\x1b[G\n\
 - tofreq{{a,b,c...}}, tolist{{{{a,b}},{{c,d}}...}}, tolist{{a,b,c}} (sorts and counts how many time each number occurs, tolist takes that kind of data and reverses it)\x1b[G\n\
-- variance/var, standarddeviation/σ (sample-bias corrected)\x1b[G\n\
+- variance/var, covariance/cov, standarddeviation/σ (sample-bias corrected)\x1b[G\n\
 - percentile({{vec}},nth) (gets number at nth percentile), percentilerank({{vec}},x) (gets percentile rank for x point), quartiles{{vec}} (gets quartiles for data set)\x1b[G\n\
 - normP(μ,σ,x) (normal distribution pdf) normD(z)/normD(x,μ,σ) (area under curve to the left of z score cdf)\x1b[G\n\
 - betaP(α,β,x) (beta distribution pdf) I(x,a,b) (regularized incomplete beta function, or beta distributions cdf)\x1b[G\n\
@@ -111,7 +111,7 @@ Functions:\x1b[G\n\
 - An(n,k), Ap(n,t) eulerian numbers and polynomials\x1b[G\n\
 - lim(x,f(x),point (,side)) both sides are checked by default, -1 for left, 1 for right\x1b[G\n\
 - slope(x,f(x),point (,nth derivitive) (,0) ), can add a 0 to the args to not combine the x and y slopes for parametric equations, same for area\x1b[G\n\
-- area(x,f(x),from,to (,0) ), length(x,f(x),from,to)\x1b[G\n\
+- area(x,f(x),from,to (,0) ), length(x,f(x),from,to), surfacearea(a,b,z(a,b),startb,endb,starta,enda)\x1b[G\n\
 - solve(x,f(x) (,point)) employs newtons method to find the root of a function at a starting point, assumes 0 if no point given, outputs Nan if newton method fails\x1b[G\n\
 - iter(x,f(x),p,n), f(x) iterated n times at point p, add \",1\" to args to show steps\x1b[G\n\n\
 Vector operations/functions:\x1b[G\n\
@@ -136,7 +136,8 @@ Matrix operations/functions:\x1b[G\n\
 - iden(n) produces an n dimension identity matrix\x1b[G\n\
 - rotate(theta), rotate(yaw,pitch,roll) produces a rotational matrix\x1b[G\n\
 - sort(mat) sorts rows by first column\x1b[G\n\
-- interpolate(mat,x) using lagrange interpolation interpolates a 2xN matrix along x, matrix should be organized like {{{{x0,y0}},{{x1,y1}} ... {{xN,yN}}}}\x1b[G\n\
+- interpolate/inter(mat,x) using lagrange interpolation interpolates a 2xN matrix along x, matrix should be organized like {{{{x0,y0}},{{x1,y1}} ... {{xN,yN}}}}\x1b[G\n\
+- lineofbestfit/lobf(mat,x) line of best fit for a matrix organized like {{{{x0,y0}},{{x1,y1}} ... {{xN,yN}}}}\x1b[G\n\
 - other functions are applied like sqrt{{{{2,4}},{{5,6}}}}={{{{sqrt(2),sqrt(4)}},{{sqrt(5),sqrt(6)}}}}\x1b[G\n\n\
 Constants:\x1b[G\n\
 - c: speed of light, 299792458 m/s\x1b[G\n\
@@ -170,38 +171,10 @@ i=>ι, I=>Ι, k=>κ, Κ=>Κ, l=>λ, L=>Λ, m=>μ, M=>Μ,\x1b[G\n\
 n=>ν, Ν=>Ν, o=>ο, O=>Ο, p=>π, P=>Π, q=>θ, Q=>Θ,\x1b[G\n\
 r=>ρ, R=>Ρ, s=>σ, S=>Σ, t=>τ, T=>Τ, u=>υ, U=>Υ,\x1b[G\n\
 w=>ω, W=>Ω, y=>ψ, Y=>Ψ, x=>χ, X=>Χ, z=>ζ, Z=>Ζ,\x1b[G\n\
-==>±, `=>ⁱ, _=>∞, ;=>°\x1b[G\n\
++=>±, ==>≈, `=>ⁱ, _=>∞, ;=>°\x1b[G\n\
 numbers/minus sign convert to superscript acting as exponents\x1b[G\n"
 ,all_units());
 }
-//TODO
-pub fn help_for(thing: &str) -> String
-{
-    match thing
-    {
-        "W" | "productlog" | "lambertw" =>
-        {
-            "W(k,z), W(z)\x1b[G\n\
-            kth branch of the inverse of z*e^z\x1b[G\n\
-            given one argument assumes k=0"
-        }
-        "atan" | "arctan" | "atan2" =>
-        {
-            "atan(y/x), atan(x,y), atan2(y,x)\x1b[G\n\
-        inverse of tan(z)\x1b[G\n\
-        using the 2 arg version gives you an angle from 0 instead of from the x axis\x1b[G\n\
-        example using cardinal directions: atan(-2,-3)=-2.15 E->N, atan(-3/-2)=0.98 W->S"
-        }
-        "->"|"to"=>"divides the left number and unit by the right unit after the '+'/'-' step of order of operations, bit more complex for fereignheit/celsius",
-        "units" => "see \"units list\" for a list of all units supported\x1b[G\nsupports metric and binary prefixes, \"units\" function extracts the units of the given input",
-        "units list" =>
-            all_units(),
-        "help" => "W, atan\x1b[G\nunits, ->",
-        "" => "",
-        _ => "not in database",
-    }
-    .to_string()
-}
 fn all_units() -> &'static str
 {
     "\"m\" | \"meter\"\x1b[G\n\
@@ -271,3 +244,44 @@ fn all_units() -> &'static str
 \"au\"\x1b[G\n\
 \"floz\""
 }
+pub fn help_for(thing: &str) -> String
+{
+    match thing
+    {
+        "W" | "productlog" | "lambertw" =>
+        {
+            "W(k,z), W(z)\x1b[G\n\
+            kth branch of the inverse of z*e^z\x1b[G\n\
+            given one argument assumes k=0"
+        }
+        "atan" | "arctan" | "atan2" =>
+        {
+            "atan(y/x), atan(x,y), atan2(y,x)\x1b[G\n\
+        inverse of tan(z)\x1b[G\n\
+        using the 2 arg version gives you an angle from 0 instead of from the x axis\x1b[G\n\
+        example using cardinal directions: atan(-2,-3)=-2.15 E->N, atan(-3/-2)=0.98 W->S"
+        }
+        "->"|"to"=>"divides the left number and unit by the right unit after the '+'/'-' step of order of operations, bit more complex for fereignheit/celsius",
+        "units" => "see \"units list\" for a list of all units supported\x1b[G\nsupports metric and binary prefixes, \"units\" function extracts the units of the given input",
+        "units list" =>
+            all_units(),
+        "help" => "W, atan\x1b[G\nunits, ->",
+        "point"|"points"=>". - dot\x1b[G\n\
++ - plus\x1b[G\n\
+x - cross\x1b[G\n\
+* - star\x1b[G\n\
+s - empty square\x1b[G\n\
+S - filled square\x1b[G\n\
+o - empty circle\x1b[G\n\
+O - filled circle\x1b[G\n\
+t - empty triangle\x1b[G\n\
+T - filled triangle\x1b[G\n\
+d - empty del (upside down triangle)\x1b[G\n\
+D - filled del (upside down triangle)\x1b[G\n\
+r - empty rhombus\x1b[G\n\
+R - filled rhombus",
+        "" => "",
+        _ => "not in database",
+    }
+    .to_string()
+}
