@@ -30,34 +30,52 @@ pub fn get_file_vars(
             {
                 l.clone()
             };
-            if if l.contains('(')
-            {
-                r.contains(&(left.clone() + "("))
-                    || r.contains(&(left.clone() + "{"))
-                    || r.contains(&(left.clone() + "["))
+            if !blacklist.contains(&l) && {
+                let mut word = String::new();
+                let mut b = false;
+                for c in r.chars()
+                {
+                    if c.is_alphanumeric() || matches!(c, '\'' | '`' | '_')
+                    {
+                        word.push(c)
+                    }
+                    else
+                    {
+                        if l.contains('(')
+                        {
+                            b = word == left && matches!(c, '(' | '{' | '[' | '|');
+                        }
+                        else
+                        {
+                            b = word == left;
+                        }
+                        if b
+                        {
+                            break;
+                        }
+                        word.clear()
+                    }
+                }
+                b
             }
-            else
-            {
-                r.contains(&left)
-            } && !blacklist.contains(&l)
             {
                 if let Some(r) = split.next()
                 {
+                    let le = l.chars().collect::<Vec<char>>();
                     blacklist.push(l.clone());
-                    let l = l.chars().collect::<Vec<char>>();
                     get_file_vars(options, vars, lines.clone(), r, blacklist);
                     for (i, j) in vars.clone().iter().enumerate()
                     {
-                        if j.name.len() <= l.len()
+                        if j.name.len() <= le.len()
                         {
-                            if let Err(s) = add_var(l, r, i, vars, options, false, false, false)
+                            if let Err(s) = add_var(le, r, i, vars, options, false, false, false)
                             {
                                 println!("{}", s)
                             }
                             continue 'lower;
                         }
                     }
-                    if let Err(s) = add_var(l, r, 0, vars, options, false, false, false)
+                    if let Err(s) = add_var(le, r, 0, vars, options, false, false, false)
                     {
                         println!("{}", s)
                     }
@@ -753,9 +771,9 @@ pub fn add_var(
                         &mut 0,
                         options,
                         false,
-                        false,
                         0,
                         s.chars().collect::<Vec<char>>(),
+                        false,
                     )?;
                     func_vars.push((-1, s.clone()));
                     fvs.push((s, parsed.0));
@@ -770,9 +788,9 @@ pub fn add_var(
             &mut 0,
             options,
             false,
-            false,
             0,
             l.clone(),
+            false,
         )?;
         parsed.1.extend(fvs);
         if l.contains(&'(')
@@ -870,9 +888,9 @@ pub fn add_var(
                                     &mut 0,
                                     options,
                                     false,
-                                    false,
                                     0,
                                     s.chars().collect::<Vec<char>>(),
+                                    false,
                                 )?;
                                 func_vars.push((-1, s.clone()));
                                 fvs.push((s, parsed.0));
@@ -887,9 +905,9 @@ pub fn add_var(
                         &mut 0,
                         options,
                         false,
-                        false,
                         0,
                         v.name.clone(),
+                        false,
                     )?;
                     parsed.1.extend(fvs);
                     if v.name.contains(&'(')
@@ -939,7 +957,7 @@ pub fn set_commands_or_vars(
     let r = split.next().unwrap();
     if l.is_empty()
         || l.chars()
-            .any(|c| !c.is_alphanumeric() && !matches!(c, '(' | ')' | ',' | '\'' | '`'))
+            .any(|c| !c.is_alphanumeric() && !matches!(c, '(' | ')' | ',' | '\'' | '`' | '_'))
     {
         return Ok(());
     }
@@ -978,6 +996,6 @@ pub fn set_commands_or_vars(
             return Ok(());
         }
     }
-    add_var(l, r, 0, vars, *options, true, false, false)?;
+    add_var(l, r, vars.len(), vars, *options, true, false, false)?;
     Ok(())
 }
