@@ -16,13 +16,14 @@ mod units;
 use crate::{
     complex::NumStr,
     graph::graph,
+    help::help_for,
     load_vars::{add_var, get_cli_vars, get_file_vars, get_vars, set_commands_or_vars},
     math::do_math,
     misc::{
-        clear, clearln, convert, get_terminal_dimensions, handle_err, prompt, read_single_char,
-        write,
+        clear, clearln, convert, get_terminal_dimensions, handle_err, insert_last, prompt,
+        read_single_char, write,
     },
-    options::{arg_opts, commands, file_opts},
+    options::{arg_opts, commands, equal_to, file_opts, silent_commands},
     parse::input_var,
     print::{print_answer, print_concurrent},
     AngleType::Radians,
@@ -134,6 +135,20 @@ pub enum AngleType
     Degrees,
     Gradians,
 }
+#[derive(Default, Copy, Clone, PartialEq)]
+pub struct HowGraphing
+{
+    graph: bool,
+    x: bool,
+    y: bool,
+}
+#[derive(Copy, Clone, PartialEq)]
+pub struct Fractions
+{
+    num: bool,
+    vec: bool,
+    mat: bool,
+}
 #[derive(Copy, Clone, PartialEq)]
 pub enum Notation
 {
@@ -157,10 +172,10 @@ pub struct Options
     angle: AngleType,
     graphtype: GraphType,
     base: (i32, i32),
-    ticks: f64,
+    ticks: (f64, f64, f64),
     onaxis: bool,
     polar: bool,
-    frac: bool,
+    frac: Fractions,
     real_time_output: bool,
     decimal_places: usize,
     color: bool,
@@ -201,10 +216,14 @@ impl Default for Options
             angle: Radians,
             graphtype: GraphType::Normal,
             base: (10, 10),
-            ticks: 20.0,
+            ticks: (20.0, 20.0, 20.0),
             onaxis: true,
             polar: false,
-            frac: true,
+            frac: Fractions {
+                num: true,
+                vec: false,
+                mat: false,
+            },
             real_time_output: true,
             decimal_places: 12,
             color: true,
@@ -321,7 +340,7 @@ fn main()
     let base = options.base;
     if !options.interactive && options.allow_vars && !options.stay_interactive
     {
-        get_cli_vars(options, args.concat(), &mut vars)
+        get_cli_vars(options, args.join(" "), &mut vars)
     }
     {
         if options.allow_vars && !default
@@ -344,7 +363,7 @@ fn main()
                     })
                     .collect::<Vec<String>>();
                 let mut split;
-                let args = args.concat();
+                let args = args.join(" ");
                 let mut blacklist = if options.interactive || options.stay_interactive
                 {
                     Vec::new()
@@ -369,30 +388,42 @@ fn main()
                     };
                     if options.interactive
                         || options.stay_interactive
-                        || (!blacklist.contains(&l)
-                            && if l.contains('(')
+                        || (!blacklist.contains(&l) && {
+                            let mut b = false;
+                            let mut word = String::new();
+                            for c in args.chars()
                             {
-                                args.contains(&(left.clone() + "("))
-                                    || args.contains(&(left.clone() + "{"))
-                                    || args.contains(&(left.clone() + "["))
-                                    || args.contains(&(left.clone() + "|"))
+                                if c.is_alphanumeric() || matches!(c, '\'' | '`' | '_')
+                                {
+                                    word.push(c)
+                                }
+                                else
+                                {
+                                    if l.contains('(')
+                                    {
+                                        b = word == left && matches!(c, '(' | '{' | '[' | '|');
+                                    }
+                                    else
+                                    {
+                                        b = word == left;
+                                    }
+                                    if b
+                                    {
+                                        break;
+                                    }
+                                    word.clear()
+                                }
                             }
-                            else
-                            {
-                                args.contains(&left)
-                            })
+                            b
+                        })
                     {
                         if let Some(r) = split.next()
                         {
                             let le = l.chars().collect::<Vec<char>>();
                             if !options.interactive && !options.stay_interactive
                             {
-                                get_file_vars(options, &mut vars, lines.clone(), r, &mut blacklist);
-                                if blacklist.contains(&l)
-                                {
-                                    continue;
-                                }
                                 blacklist.push(l);
+                                get_file_vars(options, &mut vars, lines.clone(), r, &mut blacklist);
                             }
                             for (i, v) in vars.iter().enumerate()
                             {
@@ -466,11 +497,12 @@ fn main()
     };
     let mut handles: Vec<JoinHandle<()>> = Vec::new();
     let mut cut: Vec<char> = Vec::new();
-    loop
+    'main: loop
     {
         let mut input = Vec::new();
-        let mut graphable = false;
+        let mut graphable = HowGraphing::default();
         let mut varcheck = false;
+        let mut last = Vec::new();
         if !args.is_empty()
         {
             let watch = if options.debug
@@ -484,53 +516,99 @@ fn main()
             input = args.remove(0).chars().collect();
             let output;
             let funcvar;
-            if input.contains(&'#')
+            let mut options = options;
+            let mut unparsed = input.clone();
             {
-                graphable = true
-            }
-            else
-            {
-                (output, funcvar, graphable, varcheck) = match input_var(
-                    &input.iter().map(convert).collect::<String>(),
-                    &vars,
-                    &mut Vec::new(),
-                    &mut 0,
-                    options,
-                    false,
-                    false,
-                    0,
-                    Vec::new(),
-                )
+                let split = input.split(|c| c == &';');
+                let count = split.clone().count();
+                if count != 1
                 {
-                    Ok(f) => f,
+                    unparsed = split.clone().last().unwrap().to_vec();
+                    for (i, s) in split.enumerate()
+                    {
+                        if i == count - 1
+                        {
+                            break;
+                        }
+                        silent_commands(
+                            &mut options,
+                            &s.iter()
+                                .copied()
+                                .filter(|&c| !c.is_whitespace())
+                                .collect::<Vec<char>>(),
+                        );
+                        if s.contains(&'=')
+                        {
+                            if let Err(s) =
+                                set_commands_or_vars(&mut colors, &mut options, &mut vars, s)
+                            {
+                                println!("{}", s);
+                                continue 'main;
+                            }
+                        }
+                    }
+                }
+                let tempinput = unparsed.iter().collect::<String>();
+                if tempinput.starts_with("help ")
+                {
+                    println!("{}", help_for(tempinput.splitn(2, ' ').last().unwrap()));
+                    continue;
+                }
+                else if tempinput.ends_with('=')
+                {
+                    println!(
+                        "{}",
+                        equal_to(
+                            options,
+                            &colors,
+                            &vars,
+                            &tempinput[..tempinput.len().saturating_sub(1)],
+                            "",
+                        )
+                    );
+                    continue;
+                }
+            }
+            (output, funcvar, graphable, varcheck) = match input_var(
+                &unparsed.iter().map(convert).collect::<String>(),
+                &vars,
+                &mut Vec::new(),
+                &mut 0,
+                options,
+                false,
+                0,
+                Vec::new(),
+                false,
+            )
+            {
+                Ok(f) => f,
+                Err(s) =>
+                {
+                    println!("{}: {}", input.iter().collect::<String>(), s);
+                    continue;
+                }
+            };
+            if !graphable.graph && !varcheck
+            {
+                match do_math(output, options, funcvar)
+                {
+                    Ok(n) => print_answer(n, options, &colors),
                     Err(s) =>
                     {
                         println!("{}: {}", input.iter().collect::<String>(), s);
                         continue;
                     }
-                };
-                if !graphable && !varcheck
-                {
-                    match do_math(output, options, funcvar)
-                    {
-                        Ok(n) => print_answer(n, options, &colors),
-                        Err(s) =>
-                        {
-                            println!("{}: {}", input.iter().collect::<String>(), s);
-                            continue;
-                        }
-                    }
                 }
-                if !graphable && !varcheck
+            }
+            if !graphable.graph && !varcheck
+            {
+                if let Some(time) = watch
                 {
-                    if let Some(time) = watch
-                    {
-                        println!(" {}", time.elapsed().as_nanos());
-                    }
-                    else
-                    {
-                        println!();
-                    }
+                    println!(" {}", time.elapsed().as_nanos());
+                }
+                else
+                {
+                    println!();
                 }
             }
             if args.is_empty() && options.stay_interactive
@@ -561,7 +639,7 @@ fn main()
             let mut lines = unmod_lines.clone().unwrap();
             let mut i = lines.len();
             let mut placement = 0;
-            let last = if i == 0
+            last = if i == 0
             {
                 Vec::new()
             }
@@ -614,7 +692,7 @@ fn main()
                         }
                         if !input.is_empty() && !input.starts_with(&['#'])
                         {
-                            let num = frac + if !graphable && !varcheck { 1 } else { 0 };
+                            let num = frac + if !graphable.graph && !varcheck { 1 } else { 0 };
                             if num != 0
                             {
                                 print!("\x1b[{}B", num);
@@ -636,7 +714,7 @@ fn main()
                         print!("\x1b[G\n\x1b[K");
                         if c == '\x09'
                         {
-                            graphable = false;
+                            graphable = HowGraphing::default();
                         }
                         break;
                     }
@@ -1218,6 +1296,18 @@ fn main()
                             firstslow = false;
                             placement = input.len();
                             end = input.len();
+                            start = if get_terminal_dimensions().0
+                                - if options.prompt { 3 } else { 1 }
+                                > input.len()
+                            {
+                                0
+                            }
+                            else
+                            {
+                                input.len()
+                                    - (get_terminal_dimensions().0
+                                        - if options.prompt { 3 } else { 1 })
+                            };
                             handle_err(
                                 "too slow, will print on enter",
                                 &input,
@@ -1231,17 +1321,19 @@ fn main()
                         {
                             placement = input.len();
                             end = input.len();
+                            start = if get_terminal_dimensions().0
+                                - if options.prompt { 3 } else { 1 }
+                                > input.len()
+                            {
+                                0
+                            }
+                            else
+                            {
+                                input.len()
+                                    - (get_terminal_dimensions().0
+                                        - if options.prompt { 3 } else { 1 })
+                            };
                         }
-                        start = if get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
-                            > input.len()
-                        {
-                            0
-                        }
-                        else
-                        {
-                            input.len()
-                                - (get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 })
-                        };
                         if options.real_time_output && !slow
                         {
                             execute!(stdout, DisableBlinking).unwrap();
@@ -1283,6 +1375,18 @@ fn main()
                             firstslow = false;
                             placement = input.len();
                             end = input.len();
+                            start = if get_terminal_dimensions().0
+                                - if options.prompt { 3 } else { 1 }
+                                > input.len()
+                            {
+                                0
+                            }
+                            else
+                            {
+                                input.len()
+                                    - (get_terminal_dimensions().0
+                                        - if options.prompt { 3 } else { 1 })
+                            };
                             handle_err(
                                 "too slow, will print on enter",
                                 &input,
@@ -1296,17 +1400,19 @@ fn main()
                         {
                             placement = input.len();
                             end = input.len();
+                            start = if get_terminal_dimensions().0
+                                - if options.prompt { 3 } else { 1 }
+                                > input.len()
+                            {
+                                0
+                            }
+                            else
+                            {
+                                input.len()
+                                    - (get_terminal_dimensions().0
+                                        - if options.prompt { 3 } else { 1 })
+                            };
                         }
-                        start = if get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
-                            > input.len()
-                        {
-                            0
-                        }
-                        else
-                        {
-                            input.len()
-                                - (get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 })
-                        };
                         if options.real_time_output && !slow
                         {
                             execute!(stdout, DisableBlinking).unwrap();
@@ -1397,13 +1503,17 @@ fn main()
                             }
                             if hit
                             {
-                                placement = 1;
+                                placement = 0;
                             }
                             if placement <= start
                             {
                                 end = placement
                                     + (get_terminal_dimensions().0
                                         - if options.prompt { 3 } else { 1 });
+                                if end > input.len()
+                                {
+                                    end = input.len()
+                                }
                                 start = placement;
                                 clearln(&input, start, end, options, &colors);
                                 if end - placement != 0
@@ -1447,13 +1557,14 @@ fn main()
                             }
                             if hit
                             {
-                                placement = input.len().saturating_sub(1);
+                                placement = input.len();
                             }
                             if placement >= end
                             {
-                                start = placement
-                                    - (get_terminal_dimensions().0
-                                        - if options.prompt { 3 } else { 1 });
+                                start = placement.saturating_sub(
+                                    get_terminal_dimensions().0
+                                        - if options.prompt { 3 } else { 1 },
+                                );
                                 end = placement;
                                 clearln(&input, start, end, options, &colors)
                             }
@@ -1572,10 +1683,7 @@ fn main()
                 continue;
             }
             write(
-                input
-                    .iter()
-                    .collect::<String>()
-                    .replace('_', &format!("({})", last.iter().collect::<String>())),
+                insert_last(&input, last.iter().collect::<String>().as_str()),
                 file.as_mut().unwrap(),
                 unmod_lines.as_mut().unwrap(),
                 slow,
@@ -1609,11 +1717,9 @@ fn main()
             }
             stdout.flush().unwrap()
         }
-        else if graphable
+        else if graphable.graph
         {
-            let inputs: Vec<String> = input
-                .iter()
-                .collect::<String>()
+            let inputs: Vec<String> = insert_last(&input, &last.iter().collect::<String>())
                 .split('#')
                 .map(String::from)
                 .collect();
