@@ -128,7 +128,8 @@ pub fn digraph(char: Option<char>) -> char
         '1' => '¹',
         '-' => '⁻',
         '`' => 'ⁱ',
-        '=' => '±',
+        '+' => '±',
+        '=' => '≈',
         '_' => '∞',
         '\n' => '\n',
         '\x08' => '\x08',
@@ -150,7 +151,7 @@ pub fn convert(c: &char) -> char
     let valid_chars = [
         '+', '^', '(', ')', '.', '=', ',', '#', '|', '&', '!', '%', '_', '<', '>', ' ', '[', ']',
         '{', '}', '√', '∛', '⁻', 'ⁱ', '`', '±', '∞', ';', ':', '\'', '⌊', '⌈', '⌉', '⌋', '∫', '°',
-        '$', '¢',
+        '$', '¢', '≈',
     ];
     match c
     {
@@ -495,8 +496,8 @@ pub fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
                         sum.remove(0);
                     }
                     "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
-                    | "integrate" | "arclength" | "∫" | "area" | "solve" | "length" | "slope"
-                    | "lim" | "limit" | "iter"
+                    | "integrate" | "arclength" | "∫" | "area" | "surfacearea" | "sarea"
+                    | "solve" | "length" | "slope" | "lim" | "limit" | "iter"
                         if i + 2 < func.len()
                             && if let Str(s) = &func[i + 2]
                             {
@@ -510,6 +511,20 @@ pub fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
                         i += 3;
                         sum.push(bracket)
                     }
+                    "surfacearea" | "sarea"
+                        if i + 4 < func.len()
+                            && if let Str(s) = &func[i + 4]
+                            {
+                                matches!(s.as_str(), "x" | "y")
+                            }
+                            else
+                            {
+                                false
+                            } =>
+                    {
+                        i += 5;
+                        sum.push(bracket)
+                    }
                     _ =>
                     {}
                 }
@@ -544,12 +559,18 @@ pub fn place_var(mut func: Vec<NumStr>, var: &str, num: NumStr) -> Vec<NumStr>
                     }
                     "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
                     | "integrate" | "arclength" | "∫" | "area" | "solve" | "length" | "slope"
-                    | "lim" | "limit" | "iter"
+                    | "lim" | "limit" | "iter" | "surfacearea" | "sarea"
                         if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
                     {
                         i += 3;
                         sum.push(bracket)
                     }
+                    "surfacearea" | "sarea"
+                        if i + 4 < func.len() && func[i + 4] == Str(var.to_string()) =>
+                    {
+                        i += 5;
+                        sum.push(bracket)
+                    }
                     _ =>
                     {}
                 }
@@ -664,3 +685,62 @@ pub fn parsed_to_string(
     }
     to_output(&out.chars().collect::<Vec<char>>(), options.color, colors)
 }
+pub fn insert_last(input: &[char], last: &str) -> String
+{
+    let mut output = String::new();
+    let mut word = String::new();
+    for c in input
+    {
+        if c.is_alphanumeric() || matches!(c, '\'' | '`' | '_')
+        {
+            output.push(*c);
+            word.push(*c)
+        }
+        else
+        {
+            if word.to_ascii_lowercase() == "ans"
+            {
+                output.drain(output.len() - 3..);
+                output.push('(');
+                output.push_str(last);
+                output.push(')');
+            }
+            else if word == "_"
+            {
+                output.pop();
+                output.push('(');
+                output.push_str(last);
+                output.push(')');
+            }
+            output.push(*c);
+            word.clear()
+        }
+    }
+    if word.to_ascii_lowercase() == "ans"
+    {
+        output.drain(output.len() - 3..);
+        if !last.contains('#')
+        {
+            output.push('(');
+        }
+        output.push_str(last);
+        if !last.contains('#')
+        {
+            output.push(')');
+        }
+    }
+    else if word == "_"
+    {
+        output.pop();
+        if !last.contains('#')
+        {
+            output.push('(');
+        }
+        output.push_str(last);
+        if !last.contains('#')
+        {
+            output.push(')');
+        }
+    }
+    output
+}
