@@ -3,7 +3,7 @@ use crate::{
     help::{help, help_for},
     load_vars::get_vars,
     math::do_math,
-    misc::{parsed_to_string, to_output},
+    misc::{insert_last, parsed_to_string, to_output},
     parse::input_var,
     print::get_output,
     AngleType::{Degrees, Gradians, Radians},
@@ -160,9 +160,11 @@ pub fn set_commands(
     colors: &mut Colors,
     vars: &mut [Variable],
     l: &str,
-    r: &str,
+    o: &str,
 ) -> Result<(), &'static str>
 {
+    let s = o.replace(" ", "");
+    let r = s.as_str();
     match l
     {
         "angle" =>
@@ -190,8 +192,8 @@ pub fn set_commands(
         {
             options.graphtype = match r
             {
-                "normal" => GraphType::Normal,
-                "null" | "none" => GraphType::None,
+                "normal" | "true" => GraphType::Normal,
+                "null" | "none" | "false" => GraphType::None,
                 "depth" => GraphType::Depth,
                 "flat" => GraphType::Flat,
                 _ => return Err("bad graph type"),
@@ -234,7 +236,7 @@ pub fn set_commands(
         }
         "label" =>
         {
-            let mut split = r.split(',');
+            let mut split = o.split(',');
             if split.clone().count() == 2
             {
                 colors.label.0 = split.next().unwrap().to_string();
@@ -249,33 +251,52 @@ pub fn set_commands(
                 )
             }
         }
-        "point" =>
+        "point" | "points" =>
         {
-            let r = r.chars().next().unwrap();
-            if matches!(
-                r,
-                '.' | '+' | 'x' | '*' | 's' | 'S' | 'o' | 'O' | 't' | 'T' | 'd' | 'D' | 'r' | 'R'
-            )
-            {
-                options.point_style = r;
-            }
-            else
+            if r.is_empty()
             {
                 return Err("Invalid point type");
             }
+            else
+            {
+                let r = r.chars().next().unwrap();
+                if matches!(
+                    r,
+                    '.' | '+'
+                        | 'x'
+                        | '*'
+                        | 's'
+                        | 'S'
+                        | 'o'
+                        | 'O'
+                        | 't'
+                        | 'T'
+                        | 'd'
+                        | 'D'
+                        | 'r'
+                        | 'R'
+                )
+                {
+                    options.point_style = r;
+                }
+                else
+                {
+                    return Err("Invalid point type");
+                }
+            }
         }
         "graphcli" | "slowcheck" | "interactive" | "color" | "prompt" | "surface" | "rt"
-        | "siunits" | "line" | "lines" | "polar" | "frac" | "multi" | "tabbed" | "comma"
-        | "units" | "scalegraph" | "debug" | "vars" | "onaxis" | "base" | "ticks" | "decimal"
-        | "deci" | "decimals" | "graphprec" | "graphprecision" | "prec" | "windowsize"
-        | "precision" | "range" | "xr" | "yr" | "zr" | "vrange" | "vxr" | "vyr" | "vzr" | "2d"
-        | "3d" =>
+        | "siunits" | "line" | "lines" | "polar" | "frac" | "fractions" | "fractionsv"
+        | "fractionsm" | "multi" | "tabbed" | "comma" | "units" | "scalegraph" | "debug"
+        | "vars" | "onaxis" | "base" | "ticks" | "decimal" | "deci" | "decimals" | "graphprec"
+        | "graphprecision" | "prec" | "windowsize" | "precision" | "range" | "xr" | "yr" | "zr"
+        | "vrange" | "vxr" | "vyr" | "vzr" | "2d" | "3d" =>
         {
             let mut args: Vec<Float> = Vec::new();
             {
                 let mut bracket = 0;
                 let mut last = 0;
-                for (i, c) in r.chars().enumerate()
+                for (i, c) in o.chars().enumerate()
                 {
                     match c
                     {
@@ -284,15 +305,15 @@ pub fn set_commands(
                         ',' if bracket == 0 =>
                         {
                             let parsed = input_var(
-                                &r[last..i],
+                                &o[last..i],
                                 vars,
                                 &mut Vec::new(),
                                 &mut 0,
                                 *options,
                                 false,
-                                false,
                                 0,
                                 Vec::new(),
+                                false,
                             )?;
                             args.push(
                                 do_math(parsed.0, *options, parsed.1)?
@@ -308,15 +329,15 @@ pub fn set_commands(
                     }
                 }
                 let parsed = input_var(
-                    &r[last..],
+                    &o[last..],
                     vars,
                     &mut Vec::new(),
                     &mut 0,
                     *options,
                     false,
-                    false,
                     0,
                     Vec::new(),
+                    false,
                 )?;
                 args.push(
                     do_math(parsed.0, *options, parsed.1)?
@@ -337,7 +358,9 @@ pub fn set_commands(
                 "siunits" => options.si_units = args[0] != 0.0,
                 "line" | "lines" => options.lines = args[0] != 0.0,
                 "polar" => options.polar = args[0] != 0.0,
-                "frac" => options.frac = args[0] != 0.0,
+                "frac" | "fractions" => options.frac.num = args[0] != 0.0,
+                "fractionsv" => options.frac.vec = args[0] != 0.0,
+                "fractionsm" => options.frac.mat = args[0] != 0.0,
                 "multi" => options.multi = args[0] != 0.0,
                 "tabbed" => options.tabbed = args[0] != 0.0,
                 "comma" => options.comma = args[0] != 0.0,
@@ -386,7 +409,22 @@ pub fn set_commands(
                         }
                     }
                 }
-                "ticks" => options.ticks = args[0].to_f64(),
+                "ticks" =>
+                {
+                    if args.len() == 3
+                    {
+                        options.ticks = (args[0].to_f64(), args[1].to_f64(), args[2].to_f64())
+                    }
+                    else if args.len() == 2
+                    {
+                        (options.ticks.0, options.ticks.1) = (args[0].to_f64(), args[1].to_f64())
+                    }
+                    else
+                    {
+                        let n = args[0].to_f64();
+                        options.ticks = (n, n, n);
+                    }
+                }
                 "slowcheck" =>
                 {
                     options.slowcheck = args[0]
@@ -508,9 +546,9 @@ pub fn set_commands(
                                                     &mut 0,
                                                     *options,
                                                     false,
-                                                    false,
                                                     0,
                                                     s.chars().collect::<Vec<char>>(),
+                                                    false,
                                                 )?;
                                                 func_vars.push((-1, s.clone()));
                                                 fvs.push((s, parsed.0));
@@ -525,9 +563,9 @@ pub fn set_commands(
                                         &mut 0,
                                         *options,
                                         false,
-                                        false,
                                         0,
                                         var.name.clone(),
+                                        false,
                                     )
                                     {
                                         Ok(n) => (n.0, n.1),
@@ -826,7 +864,9 @@ pub fn silent_commands(options: &mut Options, input: &[char]) -> bool
         "siunits" => options.si_units = !options.si_units,
         "line" | "lines" => options.lines = !options.lines,
         "polar" => options.polar = !options.polar,
-        "frac" => options.frac = !options.frac,
+        "frac" | "fractions" => options.frac.num = !options.frac.num,
+        "fractionsv" => options.frac.vec = !options.frac.vec,
+        "fractionsm" => options.frac.mat = !options.frac.mat,
         "multi" => options.multi = !options.multi,
         "tabbed" => options.tabbed = !options.tabbed,
         "comma" => options.comma = !options.comma,
@@ -933,11 +973,23 @@ pub fn commands(
             stdout.flush().unwrap();
             options.polar = !options.polar;
         }
-        "frac" =>
+        "frac" | "fractions" =>
         {
             print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
-            options.frac = !options.frac;
+            options.frac.num = !options.frac.num;
+        }
+        "fractionsv" =>
+        {
+            print!("\x1b[G\x1b[A\x1b[K");
+            stdout.flush().unwrap();
+            options.frac.vec = !options.frac.vec;
+        }
+        "fractionsm" =>
+        {
+            print!("\x1b[G\x1b[A\x1b[K");
+            stdout.flush().unwrap();
+            options.frac.mat = !options.frac.mat;
         }
         "multi" =>
         {
@@ -1142,15 +1194,20 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "scalegraph" => format!("{}", options.scale_graph),
         "line" => format!("{}", options.lines),
         "polar" => format!("{}", options.polar),
-        "frac" => format!("{}", options.frac),
+        "frac" | "fractions" => format!("{}", options.frac.num),
+        "fractionsv" => format!("{}", options.frac.vec),
+        "fractionsm" => format!("{}", options.frac.mat),
         "multi" => format!("{}", options.multi),
         "tabbed" => format!("{}", options.tabbed),
         "comma" => format!("{}", options.comma),
         "units" => format!("{}", options.units),
         "graphcli" => format!("{}", options.graph_cli),
-        "point" => format!("{}", options.point_style),
+        "point" | "points" => format!("{}", options.point_style),
         "base" => format!("{} {}", options.base.0, options.base.1),
-        "ticks" => format!("{}", options.ticks),
+        "ticks" => format!(
+            "x:{} y:{} z:{}",
+            options.ticks.0, options.ticks.1, options.ticks.2
+        ),
         "onaxis" => format!("{}", options.onaxis),
         "decimal" | "deci" | "decimals" => format!("{}", options.decimal_places),
         "prec" | "precision" => format!("{}", options.prec),
@@ -1223,15 +1280,15 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         _ =>
         {
             let input = input_var(
-                &l.replace('_', &format!("({})", last)),
+                &insert_last(&l.chars().collect::<Vec<char>>(), last),
                 vars,
                 &mut Vec::new(),
                 &mut 0,
                 options,
-                false,
                 true,
                 0,
                 Vec::new(),
+                false,
             );
             if let Ok(f) = input
             {
