@@ -8,12 +8,13 @@ use crate::{
     load_vars::set_commands_or_vars,
     math::do_math,
     misc::{
-        clear, get_terminal_dimensions, handle_err, no_col, parsed_to_string, prompt, to_output,
+        clear, get_terminal_dimensions, handle_err, insert_last, no_col, parsed_to_string, prompt,
+        to_output,
     },
     options::{equal_to, silent_commands},
     parse::input_var,
     AngleType::{Degrees, Gradians, Radians},
-    Colors,
+    Colors, HowGraphing,
     Notation::{LargeEngineering, Normal, Scientific, SmallEngineering},
     Number, Options, Variable,
 };
@@ -29,12 +30,12 @@ pub fn print_concurrent(
     start: usize,
     end: usize,
     long_output: bool,
-) -> (usize, bool, bool, bool)
+) -> (usize, HowGraphing, bool, bool)
 {
     if unmodified_input.starts_with(&['#']) || unmodified_input.is_empty()
     {
         clear(unmodified_input, start, end, options, &colors);
-        return (0, false, false, false);
+        return (0, HowGraphing::default(), false, false);
     }
     let mut unparsed = unmodified_input;
     {
@@ -61,7 +62,7 @@ pub fn print_concurrent(
                     if let Err(s) = set_commands_or_vars(&mut colors, &mut options, &mut vars, s)
                     {
                         handle_err(s, unmodified_input, options, &colors, start, end);
-                        return (0, false, false, false);
+                        return (0, HowGraphing::default(), false, false);
                     }
                 }
             }
@@ -96,7 +97,7 @@ pub fn print_concurrent(
                 to_output(&unmodified_input[start..end], options.color, &colors),
                 if options.color { "\x1b[0m" } else { "" },
             );
-            return (num, false, false, false);
+            return (num, HowGraphing::default(), false, false);
         }
         else if tempinput.ends_with('=')
         {
@@ -121,7 +122,7 @@ pub fn print_concurrent(
                             out,
                             if options.color { "\x1b[0m" } else { "" }
                         );
-                        (wrap, false, false, false)
+                        (wrap, HowGraphing::default(), false, false)
                     }
                     else
                     {
@@ -131,7 +132,7 @@ pub fn print_concurrent(
                             to_output(&unmodified_input[start..end], options.color, &colors),
                             if options.color { "\x1b[0m" } else { "" },
                         );
-                        (0, false, true, false)
+                        (0, HowGraphing::default(), true, false)
                     }
                 }
                 else
@@ -151,48 +152,44 @@ pub fn print_concurrent(
                         to_output(&unmodified_input[start..end], options.color, &colors),
                         if options.color { "\x1b[0m" } else { "" }
                     );
-                    (wrap - 1, false, false, false)
+                    (wrap - 1, HowGraphing::default(), false, false)
                 }
             }
             else
             {
                 clear(unmodified_input, start, end, options, &colors);
-                (0, false, false, false)
+                (0, HowGraphing::default(), false, false)
             };
         }
     }
     let input = match input_var(
-        &unparsed
-            .iter()
-            .collect::<String>()
-            .replace('_', &format!("({})", last.iter().collect::<String>())),
+        &insert_last(unparsed, last.iter().collect::<String>().as_str()),
         &vars,
         &mut Vec::new(),
         &mut 0,
         options,
         false,
-        false,
         0,
         Vec::new(),
+        false,
     )
     {
         Ok(f) => f,
         Err(s) =>
         {
             handle_err(s, unmodified_input, options, &colors, start, end);
-            return (0, false, false, false);
+            return (0, HowGraphing::default(), false, false);
         }
     };
     {
         if input.3
         {
             let n = unparsed.iter().position(|c| c == &'=').unwrap_or(0) + 1;
-            let mut input = unparsed[n..].iter().collect::<String>();
+            let mut inputs = unparsed[n..].iter().collect::<String>();
             let mut func = unparsed[..n - 1].to_vec();
             return if matches!(
                 func.iter().collect::<String>().as_str(),
-                "angle"
-                    | "re1col"
+                "re1col"
                     | "im1col"
                     | "re2col"
                     | "im2col"
@@ -204,8 +201,10 @@ pub fn print_concurrent(
                     | "im5col"
                     | "re6col"
                     | "im6col"
+                    | "angle"
                     | "notation"
                     | "graph"
+                    | "saveto"
                     | "textc"
                     | "promptc"
                     | "imagc"
@@ -213,16 +212,18 @@ pub fn print_concurrent(
                     | "unitsc"
                     | "bracketc"
                     | "label"
+                    | "point"
+                    | "points"
             )
             {
                 print!(
                     "\x1b[G\n\x1b[J{}\x1b[A\x1b[G\x1b[K{}{}{}",
-                    input,
+                    inputs,
                     prompt(options, &colors),
                     to_output(&unmodified_input[start..end], options.color, &colors),
                     if options.color { "\x1b[0m" } else { "" }
                 );
-                (1, true, false, true)
+                (1, input.2, false, true)
             }
             else
             {
@@ -236,11 +237,11 @@ pub fn print_concurrent(
                         func_vars.push((-1, i.iter().collect()));
                     }
                 }
-                if input.contains(':')
+                if inputs.contains(':')
                 {
-                    let inp = input;
+                    let inp = inputs;
                     let mut split = inp.split(':').collect::<Vec<&str>>();
-                    input = split.pop().unwrap().to_string();
+                    inputs = split.pop().unwrap().to_string();
                     for i in split
                     {
                         if i.contains('=')
@@ -251,15 +252,15 @@ pub fn print_concurrent(
                     }
                 }
                 let out = match input_var(
-                    &input,
+                    &inputs,
                     &vars,
                     &mut func_vars,
                     &mut 0,
                     options,
-                    false,
                     true,
                     0,
                     Vec::new(),
+                    false,
                 )
                 {
                     Ok(n) => parsed_to_string(n.0, n.1, &options, &colors),
@@ -268,7 +269,7 @@ pub fn print_concurrent(
                 if out.is_empty()
                 {
                     clear(unmodified_input, start, end, options, &colors);
-                    return (0, false, false, true);
+                    return (0, HowGraphing::default(), false, true);
                 }
                 let (width, height) = get_terminal_dimensions();
                 let len = no_col(&out, options.color).len();
@@ -282,7 +283,7 @@ pub fn print_concurrent(
                             out,
                             if options.color { "\x1b[0m" } else { "" }
                         );
-                        (wrap, true, false, true)
+                        (wrap, input.2, false, true)
                     }
                     else
                     {
@@ -292,7 +293,7 @@ pub fn print_concurrent(
                             to_output(&unmodified_input[start..end], options.color, &colors),
                             if options.color { "\x1b[0m" } else { "" },
                         );
-                        (0, true, true, true)
+                        (0, input.2, true, true)
                     }
                 }
                 else
@@ -312,12 +313,12 @@ pub fn print_concurrent(
                         to_output(&unmodified_input[start..end], options.color, &colors),
                         if options.color { "\x1b[0m" } else { "" }
                     );
-                    (wrap, true, false, true)
+                    (wrap, input.2, false, true)
                 }
             };
         }
     }
-    if input.2
+    if input.2.graph
     {
         return if unmodified_input.contains(&'#')
         {
@@ -333,7 +334,7 @@ pub fn print_concurrent(
                         to_output(&unmodified_input[start..end], options.color, &colors),
                         if options.color { "\x1b[0m" } else { "" }
                     );
-                    return (1, true, false, false);
+                    return (1, input.2, false, false);
                 }
                 let mut vars = vars.clone();
                 let mut options = options;
@@ -377,7 +378,7 @@ pub fn print_concurrent(
                                             start,
                                             end,
                                         );
-                                        return (0, false, false, false);
+                                        return (0, HowGraphing::default(), false, false);
                                     }
                                 }
                             }
@@ -394,7 +395,7 @@ pub fn print_concurrent(
                 }
             }
             out.pop();
-            let (width, height) = get_terminal_dimensions();
+            let width = get_terminal_dimensions().0;
             let no_col = no_col(&out, options.color);
             let wrap = no_col
                 .split(|c| c == &'\n')
@@ -411,27 +412,15 @@ pub fn print_concurrent(
                 .sum();
             let len = no_col.len();
             let out = out.replace('\n', "\x1b[G\n");
-            if len > width * (height - 1)
+            if len > width
             {
-                if long_output
-                {
-                    print!(
-                        "\x1b[G\n\x1b[J{}{}",
-                        out,
-                        if options.color { "\x1b[0m" } else { "" }
-                    );
-                    (wrap, true, false, false)
-                }
-                else
-                {
-                    print!(
-                        "\x1b[J\x1b[G\ntoo long, will print on enter\x1b[G\x1b[A\x1b[K{}{}{}",
-                        prompt(options, &colors),
-                        to_output(&unmodified_input[start..end], options.color, &colors),
-                        if options.color { "\x1b[0m" } else { "" },
-                    );
-                    (0, true, true, false)
-                }
+                print!(
+        "\x1b[J\x1b[G\ntoo long, append '=' to see parsed input\x1b[G\x1b[A\x1b[K{}{}{}",
+        prompt(options, &colors),
+        to_output(&unmodified_input[start..end], options.color, &colors),
+        if options.color { "\x1b[0m" } else { "" },
+    );
+                (1, input.2, true, false)
             }
             else
             {
@@ -450,43 +439,31 @@ pub fn print_concurrent(
                     to_output(&unmodified_input[start..end], options.color, &colors),
                     if options.color { "\x1b[0m" } else { "" }
                 );
-                (wrap, true, false, false)
+                (wrap, input.2, false, false)
             }
         }
         else
         {
-            let input = unparsed.iter().collect::<String>();
+            let inputs = unparsed.iter().collect::<String>();
             let out = equal_to(
                 options,
                 &colors,
                 &vars,
-                &input,
+                &inputs,
                 &last.iter().collect::<String>(),
             );
-            let (width, height) = get_terminal_dimensions();
+            let width = get_terminal_dimensions().0;
             let len = no_col(&out, options.color).len();
             let wrap = (len - 1) / width + 1;
-            if len > width * (height - 1)
+            if len > width
             {
-                if long_output
-                {
-                    print!(
-                        "\x1b[G\n\x1b[J{}{}",
-                        out,
-                        if options.color { "\x1b[0m" } else { "" }
-                    );
-                    (wrap, true, false, false)
-                }
-                else
-                {
-                    print!(
-                        "\x1b[J\x1b[G\ntoo long, will print on enter\x1b[G\x1b[A\x1b[K{}{}{}",
+                print!(
+                        "\x1b[J\x1b[G\ntoo long, append '=' to see parsed input\x1b[G\x1b[A\x1b[K{}{}{}",
                         prompt(options, &colors),
                         to_output(&unmodified_input[start..end], options.color, &colors),
                         if options.color { "\x1b[0m" } else { "" },
                     );
-                    (0, true, true, false)
-                }
+                (1, input.2, true, false)
             }
             else
             {
@@ -505,7 +482,7 @@ pub fn print_concurrent(
                     to_output(&unmodified_input[start..end], options.color, &colors),
                     if options.color { "\x1b[0m" } else { "" }
                 );
-                (wrap, true, false, false)
+                (wrap, input.2, false, false)
             }
         };
     }
@@ -522,7 +499,7 @@ pub fn print_concurrent(
             {
                 handle_err(s, unmodified_input, options, &colors, start, end);
             }
-            return (0, false, false, false);
+            return (0, HowGraphing::default(), false, false);
         }
     };
     let mut frac = 0;
@@ -532,11 +509,11 @@ pub fn print_concurrent(
         Num(n) =>
         {
             let mut output = get_output(options, &colors, &n);
-            let (mut frac_a, frac_b) = if options.frac
+            let (mut frac_a, frac_b) = if options.frac.num
             {
                 let n = n.number;
-                let fa = fraction(n.real().clone(), options);
-                let fb = fraction(n.imag().clone(), options);
+                let fa = fraction(n.real().clone(), options, &colors);
+                let fb = fraction(n.imag().clone(), options, &colors);
                 let sign = if !output.0.is_empty() && n.imag().is_sign_positive()
                 {
                     "+"
@@ -645,7 +622,7 @@ pub fn print_concurrent(
             }
             let len1 = no_col(&output.0, options.color).len();
             let len2 = no_col(&output.1, options.color).len();
-            if (frac == 1 && !options.frac)
+            if (frac == 1 && !options.frac.num)
                 || (frac_a.len() + frac_b.len()
                     - if options.color && !frac_b.is_empty()
                     {
@@ -788,13 +765,15 @@ pub fn print_concurrent(
             let mut frac_out = if options.polar { "[" } else { "{" }.to_string();
             let mut out;
             let mut frac_temp;
+            let (width, height) = get_terminal_dimensions();
+            frac = if options.frac.vec { 1 } else { 0 };
             for (k, i) in v.iter().enumerate()
             {
                 out = get_output(options, &colors, i);
                 let i = &i.number;
-                if options.frac
+                if frac == 1
                 {
-                    frac_temp = fraction(i.real().clone(), options);
+                    frac_temp = fraction(i.real().clone(), options, &colors);
                     frac_out += if !frac_temp.is_empty()
                     {
                         &frac_temp
@@ -803,7 +782,7 @@ pub fn print_concurrent(
                     {
                         &out.0
                     };
-                    frac_temp = fraction(i.imag().clone(), options);
+                    frac_temp = fraction(i.imag().clone(), options, &colors);
                     frac_out += &if !frac_temp.is_empty()
                     {
                         format!(
@@ -840,6 +819,10 @@ pub fn print_concurrent(
                     {
                         out.clone().1
                     };
+                    if frac_out.len() > width
+                    {
+                        frac = 0
+                    }
                 }
                 output += &out.0;
                 output += &out.1;
@@ -864,13 +847,9 @@ pub fn print_concurrent(
                     frac_out += ",";
                 }
             }
-            let (width, height) = get_terminal_dimensions();
             let length = no_col(&output, options.color).len();
-            if frac_out != output
-            {
-                frac = 1;
-            }
-            if (frac == 1 && !options.frac)
+            frac = if frac_out != output { 1 } else { 0 };
+            if (frac == 1 && !options.frac.vec)
                 || no_col(&frac_out, options.color).len() > width
                 || length > width
             {
@@ -883,7 +862,7 @@ pub fn print_concurrent(
                     let num = (length - 1) / width;
                     print!(
                         "\x1b[G\n\x1b[J{}\x1b[G{}{}",
-                        if frac == 1 && options.frac
+                        if frac == 1 && options.frac.vec
                         {
                             format!("{}\x1b[G\n", frac_out)
                         }
@@ -946,6 +925,8 @@ pub fn print_concurrent(
             let mut out;
             let mut frac_temp;
             let mut num = 0;
+            let (width, height) = get_terminal_dimensions();
+            frac = if options.frac.mat { 1 } else { 0 };
             for (l, j) in v.iter().enumerate()
             {
                 if !options.multi
@@ -957,9 +938,9 @@ pub fn print_concurrent(
                 {
                     out = get_output(options, &colors, i);
                     let i = &i.number;
-                    if options.frac
+                    if frac == 1
                     {
-                        frac_temp = fraction(i.real().clone(), options);
+                        frac_temp = fraction(i.real().clone(), options, &colors);
                         frac_out += if !frac_temp.is_empty()
                         {
                             &frac_temp
@@ -968,7 +949,7 @@ pub fn print_concurrent(
                         {
                             &out.0
                         };
-                        frac_temp = fraction(i.imag().clone(), options);
+                        frac_temp = fraction(i.imag().clone(), options, &colors);
                         frac_out += &if !frac_temp.is_empty()
                         {
                             format!(
@@ -1005,6 +986,10 @@ pub fn print_concurrent(
                         {
                             out.clone().1
                         };
+                        if frac_out.len() > width
+                        {
+                            frac = 0
+                        }
                     }
                     output += &out.0;
                     output += &out.1;
@@ -1057,7 +1042,6 @@ pub fn print_concurrent(
                 output += "}";
                 frac_out += "}";
             }
-            let (width, height) = get_terminal_dimensions();
             let length = no_col(&output, options.color).len().saturating_sub(1);
             if frac_out != output
             {
@@ -1066,7 +1050,7 @@ pub fn print_concurrent(
             if !options.multi
             {
                 num += (length - 1) / width;
-                if (frac == 1 && !options.frac)
+                if (frac == 1 && !options.frac.mat)
                     || no_col(&frac_out, options.color).len() > width
                     || length > width
                 {
@@ -1111,7 +1095,7 @@ pub fn print_concurrent(
                 {
                     print!(
                         "\x1b[G\n\x1b[J{}\x1b[G{}{}",
-                        if frac == 1 && options.frac
+                        if frac == 1 && options.frac.mat
                         {
                             num *= 2;
                             if options.multi
@@ -1143,7 +1127,7 @@ pub fn print_concurrent(
             }
             else
             {
-                if !options.frac
+                if !options.frac.mat
                 {
                     frac = 0;
                 }
@@ -1181,7 +1165,7 @@ pub fn print_concurrent(
         }
         _ => handle_err("str err", unmodified_input, options, &colors, start, end),
     }
-    (frac, false, long, false)
+    (frac, HowGraphing::default(), long, false)
 }
 pub fn print_answer(num: NumStr, options: Options, colors: &Colors)
 {
@@ -1629,7 +1613,7 @@ pub fn get_output(
                         "i".to_string()
                     }
             },
-            if options.units && num.imag().is_zero()
+            if options.units && im == "0"
             {
                 units.map(|units| {
                     format!(
@@ -1736,6 +1720,7 @@ fn to_string(num: &Float, decimals: usize, imag: bool, radix: i32) -> String
     }
     if radix == 10
         && d.to_string().trim_end_matches('0') == "1"
+        && d.to_string().len() > decimals
         && r.trim_start_matches('0')
             .trim_start_matches('.')
             .starts_with('9')
@@ -1746,7 +1731,7 @@ fn to_string(num: &Float, decimals: usize, imag: bool, radix: i32) -> String
                 .unwrap()
                 .complete(num.prec())
                 + 1;
-            l = t.to_integer().unwrap().to_string();
+            l = t.to_integer().unwrap_or_default().to_string();
             d = Integer::new();
         }
         else
